"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.sqliteClient = exports.SqliteClient = exports.Pagination = void 0;
const fs_1 = __importDefault(require("fs"));
const util_1 = require("../util");
const better_sqlite3_1 = __importDefault(require("better-sqlite3"));
const constants_1 = require("../util/constants");
const PAGINATION = " LIMIT @index, @size ";
class Pagination {
    constructor(size = 10, page = 0) {
        this.size = size;
        this.page = page;
    }
    getQueryStartIndex() {
        return this.page * this.size;
    }
}
exports.Pagination = Pagination;
class SqliteClient {
    constructor(fileName = constants_1.DATABASE.DB_FILE_NAME) {
        this.filePath = fileName;
    }
    getFilePath() {
        return this.filePath;
    }
    createInstance() {
        const config = {};
        if (process.env.LOOPER_DEBUG) {
            config.verbose = (message) => {
                util_1.logger.debug(message);
            };
        }
        fs_1.default.mkdirSync(constants_1.DATABASE.DB_FOLDER_PATH, { recursive: true });
        return new better_sqlite3_1.default(this.filePath, config);
    }
    init(custom) {
        if (!fs_1.default.existsSync(this.filePath)) {
            const db = this.createInstance();
            const ddl = custom ? custom : constants_1.DATABASE.DDL;
            db.exec(ddl);
            db.close();
        }
    }
    createId() {
        return util_1.uuid.generate();
    }
    insert(sql, rows) {
        const db = this.createInstance();
        const insert = db.prepare(sql);
        const insertMany = db.transaction((records) => {
            for (const record of records)
                insert.run(record);
        });
        insertMany(rows);
        db.close();
    }
    insertOne(table, row) {
        const db = this.createInstance();
        const fields = Object.getOwnPropertyNames(row)
            .map((propertyName) => propertyName)
            .join(", ");
        const values = Object.getOwnPropertyNames(row)
            .map((propertyName) => `@${propertyName}`)
            .join(", ");
        const sql = `INSERT INTO ${table} (${fields}) VALUES (${values})`;
        const insert = db.prepare(sql);
        insert.run(row);
        db.close();
    }
    update(table, params, conditions) {
        const db = this.createInstance();
        const fields = Object.getOwnPropertyNames(params)
            .map((param) => `${param} = @${param}`)
            .join(", ");
        const values = Object.getOwnPropertyNames(conditions)
            .map((param) => `${param} = @${param}`)
            .join(" AND ");
        db.prepare(`UPDATE ${table} SET ${fields} WHERE ${values}`).run(Object.assign(Object.assign({}, params), conditions));
        db.close();
    }
    select(sql, params, pagination) {
        let query;
        if (pagination) {
            query = `${sql} ${PAGINATION}`;
            params.size = pagination.size;
            params.index = pagination.getQueryStartIndex();
        }
        else {
            query = sql;
        }
        return this.runSelectQuery(query, params);
    }
    selectAll(table, pagination) {
        let query;
        const params = {};
        if (pagination) {
            query = `SELECT * FROM ${table} ${PAGINATION}`;
            params.size = pagination.size;
            params.index = pagination.getQueryStartIndex();
        }
        else {
            query = `SELECT * FROM ${table}`;
        }
        return this.runSelectQuery(query, params);
    }
    existsById(table, id) {
        const query = `SELECT 1 FROM ${table} WHERE id = @id`;
        const rows = this.runSelectQuery(query, { id });
        return rows.length > 0;
    }
    findById(table, id) {
        const query = `SELECT * FROM ${table} WHERE id = @id`;
        const rows = this.runSelectQuery(query, { id });
        return rows.length > 0 ? rows[0] : undefined;
    }
    findByLikeOr(table, params, pagination) {
        const conditions = Object.getOwnPropertyNames(params)
            .map((name) => `${name} LIKE @${name}`)
            .join(" OR ");
        if (pagination) {
            params.size = pagination.size;
            params.index = pagination.getQueryStartIndex();
        }
        const query = `SELECT * FROM ${table} WHERE ${conditions} ${pagination ? PAGINATION : ""}`;
        return this.runSelectQuery(query, params);
    }
    countByLikeOr(table, params) {
        let where = "";
        if (params && Object.getOwnPropertyNames(params).length > 0) {
            where = ` WHERE ${Object.getOwnPropertyNames(params)
                .map((name) => `${name} LIKE @${name}`)
                .join(" OR ")}`;
        }
        const query = `SELECT count(1) as count FROM ${table} ${where}`;
        const rows = this.runSelectQuery(query, params);
        return rows[0].count;
    }
    count(table, params) {
        let where = "";
        if (params && Object.getOwnPropertyNames(params).length > 0) {
            where = `WHERE ${Object.getOwnPropertyNames(params)
                .map((param) => `${param} = @${param}`)
                .join(" AND ")}`;
        }
        const query = `SELECT count(1) as count FROM ${table} ${where}`;
        const rows = this.runSelectQuery(query, params);
        return rows[0].count;
    }
    runSelectQuery(query, params) {
        const db = this.createInstance();
        const stmt = db.prepare(query);
        const rows = params ? stmt.all(params) : stmt.all();
        db.close();
        return rows;
    }
    run(sql, params) {
        const db = this.createInstance();
        const stmt = db.prepare(sql);
        stmt.run(params);
        db.close();
    }
    deleteById(table, id) {
        this.run(`DELETE FROM ${table} WHERE id = @id`, { id });
    }
}
exports.SqliteClient = SqliteClient;
exports.sqliteClient = new SqliteClient();
