"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.dataService = exports.DataService = void 0;
const sqlite_client_1 = require("../model/sqlite-client");
const storage_module_1 = require("../module/storage-module");
const sync_with_backup_task_1 = require("../tasks/sync-with-backup-task");
const constants_1 = require("../util/constants");
const mime_types_1 = require("mime-types");
const logger_1 = __importDefault(require("../util/logger"));
const media_model_1 = require("../model/media-model");
const MAX_BACKUP_KEYS = 25;
const INSERT_MEDIA = "INSERT INTO media (id, name, content_type, object_key, created_at, state, size, thumbnail_key) VALUES (@id, @name, @content_type, @object_key, @created_at, @state, @size, @thumbnail_key)";
class DataService {
    constructor(sqlite = sqlite_client_1.sqliteClient, storageM = storage_module_1.storageModule, syncTask = sync_with_backup_task_1.syncWithBackupTask) {
        this.storageModule = storageM;
        this.syncWithBackupTask = syncTask;
        this.sqliteClient = sqlite;
    }
    backup() {
        const objectKey = `db/${new Date().getTime()}`;
        return this.storageModule.upload(`${objectKey}`, constants_1.CONTENT_TYPE.APPLICATION_OCTET_STREAM, constants_1.DATABASE.DB_FILE_NAME);
    }
    downloadBackup(objectKey) {
        return __awaiter(this, void 0, void 0, function* () {
            return {
                url: yield this.storageModule.getSignedUrl(objectKey),
            };
        });
    }
    listBackups(continuationToken) {
        return __awaiter(this, void 0, void 0, function* () {
            const data = yield this.storageModule.list("db", MAX_BACKUP_KEYS, continuationToken);
            const contents = [];
            if (data.contents) {
                data.contents.forEach((content) => {
                    const saveDate = parseInt(content.Key.replace("db/", ""));
                    contents.push({
                        date: saveDate,
                        objectKey: content.Key,
                    });
                });
            }
            return {
                count: data.count,
                continuationToken: data.continuationToken,
                contents,
            };
        });
    }
    deleteBackup(objectKey) {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.storageModule.delete(objectKey);
        });
    }
    syncBackup(objectKey) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                yield this.storageModule.download(constants_1.DEFAULT_DOWNLOADED_BACKUP_DB, objectKey);
                return yield this.syncWithBackupTask.run();
            }
            catch (e) {
                logger_1.default.error("unexpected error during db sync with a backup");
                logger_1.default.error(e);
                throw constants_1.ERROR.UNEXPECTED_ERROR;
            }
        });
    }
    syncStorage() {
        return __awaiter(this, void 0, void 0, function* () {
            const foundObjects = new Map();
            let continuationToken = undefined;
            do {
                const data = yield this.storageModule.list("looper", undefined, continuationToken);
                continuationToken = data.continuationToken;
                if (data.contents) {
                    data.contents.forEach((content) => {
                        const splitKey = content.Key.split("/");
                        const id = splitKey[1];
                        const ext = splitKey[2].split(".")[1];
                        const isMedia = splitKey[2].includes("media");
                        const isThumb = splitKey[2].includes("thumbnail");
                        const size = content.Size;
                        if (foundObjects.has(id)) {
                            const media = foundObjects.get(id);
                            if (isMedia) {
                                media.object_key = content.Key;
                                media.content_type = (0, mime_types_1.lookup)(ext);
                                media.size = size;
                            }
                            if (isThumb)
                                media.thumbnail_key = content.Key;
                        }
                        else {
                            if (!this.sqliteClient.existsById(constants_1.DATABASE.TABLE.MEDIA, id)) {
                                const media = {
                                    id,
                                    name: id,
                                    thumbnail_key: undefined,
                                    state: media_model_1.MediaStateEnum.UNDER_ANALYSIS,
                                    created_at: new Date().getTime(),
                                };
                                if (isMedia) {
                                    media.object_key = content.Key;
                                    media.size = size;
                                    media.content_type = (0, mime_types_1.lookup)(ext);
                                }
                                if (isThumb) {
                                    media.thumbnail_key = content.Key;
                                }
                                foundObjects.set(id, media);
                            }
                        }
                    });
                }
            } while (continuationToken);
            if (foundObjects.size > 0) {
                this.sqliteClient.insert(INSERT_MEDIA, Array.from(foundObjects.values()));
            }
            return {
                amountMedias: foundObjects.size,
            };
        });
    }
}
exports.DataService = DataService;
exports.dataService = new DataService();
