"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.mediaService = exports.MediaService = void 0;
const media_model_1 = require("../model/media-model");
const storage_module_1 = require("../module/storage-module");
const sqlite_client_1 = require("../model/sqlite-client");
const constants_1 = require("../util/constants");
const util_1 = require("../util");
const mime_types_1 = require("mime-types");
class MediaService {
    constructor(mediaM = media_model_1.mediaModel, storage = storage_module_1.storageModule) {
        this.mediaModel = mediaM;
        this.storageModule = storage;
    }
    findById(id) {
        return this.mediaModel.findById(id);
    }
    findMedia(request, pagination = new sqlite_client_1.Pagination()) {
        return __awaiter(this, void 0, void 0, function* () {
            const data = this.mediaModel.findByNameAndState(pagination, request.name, request.status);
            if (data.records) {
                for (const media of data.records) {
                    if (media.thumbnail_key && media.thumbnail_key !== "none") {
                        media.thumbnail = yield this.storageModule.getSignedUrl(media.thumbnail_key);
                    }
                }
                data.records.sort((a, b) => a.name.localeCompare(b.name));
            }
            return data;
        });
    }
    getSignedUrl(id) {
        return __awaiter(this, void 0, void 0, function* () {
            const media = this.mediaModel.findById(id);
            if (!media) {
                throw constants_1.ERROR.NOT_FOUND;
            }
            return {
                url: yield this.storageModule.getSignedUrl(media.object_key),
                contentType: media.content_type,
            };
        });
    }
    update(id, fields) {
        this.mediaModel.update(fields, id);
    }
    deleteMedia(id) {
        return __awaiter(this, void 0, void 0, function* () {
            const media = this.mediaModel.findById(id);
            if (media) {
                yield this.storageModule.delete(media.object_key);
                if (media.thumbnail_key && media.thumbnail_key !== "none")
                    yield this.storageModule.delete(media.thumbnail_key);
                this.mediaModel.delete(id);
            }
        });
    }
    createMedia(media) {
        this.mediaModel.insert(media);
    }
    createMediaWithSingleUpload(request) {
        return __awaiter(this, void 0, void 0, function* () {
            const id = util_1.uuid.generate();
            const objectKey = this.createMediaAssetKey(id, request.contentType);
            const uploadUrl = yield this.storageModule.getSinglePartUpload(objectKey, request.contentType);
            this.createMedia({
                id: id,
                name: request.fileName,
                content_type: request.contentType,
                size: request.size,
                object_key: objectKey,
                state: media_model_1.MediaStateEnum.UPLOAD,
                created_at: new Date().getTime(),
            });
            return {
                mediaId: id,
                uploadUrl,
            };
        });
    }
    createMediaWithMultipartUpload(request) {
        return __awaiter(this, void 0, void 0, function* () {
            const id = util_1.uuid.generate();
            const objectKey = this.createMediaAssetKey(id, request.contentType);
            const createMultipartResponse = yield this.storageModule.createMultipartUpload(objectKey, request.contentType);
            this.createMedia({
                id: id,
                name: request.fileName,
                content_type: request.contentType,
                size: request.size,
                object_key: objectKey,
                state: media_model_1.MediaStateEnum.UPLOAD,
                created_at: new Date().getTime(),
            });
            return {
                mediaId: id,
                key: objectKey,
                uploadId: createMultipartResponse.UploadId,
            };
        });
    }
    getPresignedUploadPartUrl(uploadId, key, index) {
        return this.storageModule.getPresignedUploadPartUrl(uploadId, key, index);
    }
    createMediaAssetKey(mediaId, contentType) {
        return `looper/${mediaId}/media.${(0, mime_types_1.extension)(contentType)}`;
    }
    completeMediaUpload(id, multipartData) {
        return __awaiter(this, void 0, void 0, function* () {
            this.mediaModel.update({
                state: media_model_1.MediaStateEnum.UNDER_ANALYSIS,
            }, id);
            if (multipartData && multipartData.uploadId) {
                yield this.storageModule.completeMultiPartUpload(multipartData.objectKey, multipartData.uploadId, multipartData.uploadPartResults);
            }
        });
    }
    abortMediaUpload(id, multipartData) {
        this.mediaModel.update({
            state: media_model_1.MediaStateEnum.UPLOAD_FAILED,
        }, id);
        if (multipartData && multipartData.uploadId) {
            this.storageModule.abortMultiPartUpload(multipartData.objectKey, multipartData.uploadId);
        }
    }
}
exports.MediaService = MediaService;
exports.mediaService = new MediaService();
