"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.streamingService = exports.StreamingService = exports.TransmissionStateEnum = void 0;
const playlist_service_util_1 = require("./playlist-service-util");
const output_model_1 = require("../model/output-model");
const config_model_1 = require("../model/config-model");
const streaming_task_1 = require("../tasks/streaming-task");
const constants_1 = require("../util/constants");
var TransmissionStateEnum;
(function (TransmissionStateEnum) {
    TransmissionStateEnum["WAITING"] = "WAITING";
    TransmissionStateEnum["TRANSMITTING"] = "TRANSMITTING";
})(TransmissionStateEnum || (exports.TransmissionStateEnum = TransmissionStateEnum = {}));
const EXIT_SUCCESS = 0;
const DEFAULT_STATE_VALUES = {
    playlist: undefined,
    medias: undefined,
    output: undefined,
    loop: false,
    forceScale: undefined,
    transmissionState: TransmissionStateEnum.WAITING,
    startedAt: undefined,
};
class StreamingService {
    constructor() {
        this.state = Object.assign({}, DEFAULT_STATE_VALUES);
    }
    start(playlistId, outputId, forceScale, loop) {
        return __awaiter(this, void 0, void 0, function* () {
            this.state.medias = yield this.setPlaylist(playlistId);
            const output = this.setOutput(outputId);
            this.state.loop = loop;
            this.state.forceScale = forceScale;
            this.state.startedAt = new Date().getTime();
            this.run(this.state.medias, output);
        });
    }
    stop() {
        streaming_task_1.streamingTask.stop();
        this.cleanUp();
    }
    getLog() {
        return streaming_task_1.streamingTask.getLog();
    }
    getStatus() {
        return this.state;
    }
    isTransmitting() {
        return this.state.transmissionState === TransmissionStateEnum.TRANSMITTING;
    }
    isWaiting() {
        return this.state.transmissionState === TransmissionStateEnum.WAITING;
    }
    setPlaylist(playlistId) {
        return __awaiter(this, void 0, void 0, function* () {
            const playlist = yield playlist_service_util_1.playlistServiceUtil.findById(playlistId);
            if (!playlist.medias || playlist.medias.length <= 0)
                throw constants_1.ERROR.EMPTY_PLAYLIST;
            this.state.playlist = playlist;
            this.state.medias = this.createMediaList(playlist);
            return this.state.medias;
        });
    }
    createMediaList(playlist) {
        const medias = [];
        if (playlist.opening_media_data)
            medias.push(playlist.opening_media_data);
        if (playlist.medias) {
            playlist.medias.forEach((media) => {
                medias.push(media);
                if (playlist.break_playlist_data &&
                    playlist.break_playlist_data.medias) {
                    medias.push(...playlist.break_playlist_data.medias);
                }
            });
        }
        if (playlist.closing_media_data)
            medias.push(playlist.closing_media_data);
        return medias;
    }
    setOutput(outputId) {
        const output = output_model_1.outputModel.findById(outputId);
        if (!output)
            throw constants_1.ERROR.NOT_FOUND;
        this.state.output = output;
        return output;
    }
    cleanUp() {
        this.state = Object.assign({}, DEFAULT_STATE_VALUES);
    }
    onTaskComplete(code) {
        if (this.state.loop &&
            code == EXIT_SUCCESS &&
            this.state.medias &&
            this.state.output) {
            this.run(this.state.medias, this.state.output);
        }
        else {
            this.cleanUp();
        }
    }
    useCodec(medias) {
        return medias.every((media) => this.getMediaCodec(media) === this.getMediaCodec(medias[0]));
    }
    getMediaCodec(media) {
        const vCodec = media.video_codec ? media.video_codec : "?";
        const aCodec = media.audio_codec ? media.audio_codec : "?";
        const objectKeyParts = media.object_key.split(".");
        const container = objectKeyParts[objectKeyParts.length - 1];
        return `${vCodec}/${aCodec}/${container}`;
    }
    getScale(medias, forceScale) {
        if (forceScale) {
            return forceScale;
        }
        return medias[0].scale;
    }
    run(medias, output, forceScale) {
        const scale = this.getScale(medias, forceScale);
        const codec = scale ? true : this.useCodec(medias);
        const taskSettings = config_model_1.configModel.getTasksSettings();
        streaming_task_1.streamingTask.run({
            mediaList: medias,
            codec,
            scale,
            useSystemFFmpeg: taskSettings.transcoder.useSystemFFmpeg,
            outputUrl: output.url,
            onTaskComplete: (code) => this.onTaskComplete(code),
        });
        this.state.transmissionState = TransmissionStateEnum.TRANSMITTING;
    }
}
exports.StreamingService = StreamingService;
exports.streamingService = new StreamingService();
