"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.transcoderTask = exports.TranscoderTask = exports.PATH = void 0;
const os_1 = __importDefault(require("os"));
const fs_1 = __importDefault(require("fs"));
const index_1 = require("../util/index");
const ffmpeg_module_1 = require("../module/ffmpeg-module");
const console_module_1 = require("../module/console-module");
const media_model_1 = require("../model/media-model");
const storage_module_1 = require("../module/storage-module");
exports.PATH = {
    DOWNLOAD_PATH: `${os_1.default.tmpdir()}/download`,
    OUTPUT_PATH: `${os_1.default.tmpdir()}/output`,
};
class TranscoderTask extends console_module_1.ConsoleModule {
    constructor(mediaM = media_model_1.mediaModel, storageM = storage_module_1.storageModule, ffmpegM = ffmpeg_module_1.ffmpegModule) {
        super();
        this.mediaModel = mediaM;
        this.storageModule = storageM;
        this.ffmpegModule = ffmpegM;
    }
    run(settings, onComplete) {
        return __awaiter(this, void 0, void 0, function* () {
            if (settings.on) {
                const media = this.mediaModel.findFirstBySendToTranscoder();
                if (media) {
                    this.info(`Media ${media.name} download started`);
                    yield this.storageModule.download(exports.PATH.DOWNLOAD_PATH, media.object_key);
                    this.info(`Media ${media.name} download completed. Starting the process`);
                    this.transcode(media, settings.useSystemFFmpeg, onComplete);
                    return true;
                }
            }
            return false;
        });
    }
    transcode(media, useSystemFFmpeg = false, onComplete) {
        this.ffmpegModule.transcode({
            input: exports.PATH.DOWNLOAD_PATH,
            output: exports.PATH.OUTPUT_PATH,
            useSystemFFmpeg,
            onComplete: (err) => this.onTranscoderComplete(err, media, onComplete),
        });
    }
    onTranscoderComplete(err, media, onComplete) {
        if (err) {
            this.error(`transcoder finished with error code: ${err} for media ${media.name}`);
            media_model_1.mediaModel.update({
                send_to_transcoder: 0,
                was_transcoded: 1,
            }, media.id);
            onComplete();
            return;
        }
        this.completeTranscode(media, onComplete);
    }
    completeTranscode(media, onComplete) {
        return __awaiter(this, void 0, void 0, function* () {
            const id = index_1.uuid.generate();
            const objectKey = `looper/${id}/media.mp4`;
            try {
                const stats = fs_1.default.statSync(exports.PATH.OUTPUT_PATH);
                yield this.storageModule.upload(objectKey, `video/mp4`, exports.PATH.OUTPUT_PATH);
                this.mediaModel.insert({
                    id: id,
                    name: `(TRANSCODED) ${media.name}`,
                    content_type: "video/mp4",
                    object_key: objectKey,
                    size: stats.size,
                    state: media_model_1.MediaStateEnum.UNDER_ANALYSIS,
                    transcoded_at: new Date().getTime(),
                    created_at: new Date().getTime(),
                });
                this.info(`transcoder finished for media ${media.name}`);
            }
            catch (e) {
                this.error("Transcoder finished with fatal error saving media status on database.");
                this.error(e);
            }
            finally {
                this.mediaModel.update({
                    send_to_transcoder: 0,
                    was_transcoded: 1,
                }, media.id);
                this.cleanUp();
                onComplete();
            }
        });
    }
    cleanUp() {
        if (fs_1.default.existsSync(exports.PATH.DOWNLOAD_PATH))
            fs_1.default.unlinkSync(exports.PATH.DOWNLOAD_PATH);
        if (fs_1.default.existsSync(exports.PATH.OUTPUT_PATH))
            fs_1.default.unlinkSync(exports.PATH.OUTPUT_PATH);
    }
}
exports.TranscoderTask = TranscoderTask;
exports.transcoderTask = new TranscoderTask();
